
import 'dart:async';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

import '../../../model/chatList/chatListProps.dart';
import '../../../storage/base/settingsProvider.dart';
import 'chatTile.dart';
import 'joinChat.dart';
import 'searchChat.dart';

class ChatList extends StatefulWidget {
  const ChatList({Key? key}) : super(key: key);

  @override
  State<ChatList> createState() => _ChatListState();
}

class _ChatListState extends State<ChatList> {

  @override
  void initState() {
    super.initState();

    WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
      _query();
    });
  }

  void _query({bool renew = false}) {
    Provider.of<ChatListProps>(context, listen: false).run(renew: renew);
  }

  @override
  Widget build(BuildContext context) {
    ChatListProps? latestData;

    return Scaffold(
      appBar: AppBar(
        title: const Text("Talk"),
        actions: [
          IconButton(
            icon: const Icon(Icons.search),
            onPressed: () async {
              if(latestData == null) return;
              showSearch(context: context, delegate: SearchChat(latestData!.getRoomsResponse.data.toList()));
            },
          )
        ],
      ),
      floatingActionButton: FloatingActionButton(
        heroTag: "createChat",
        backgroundColor: Theme.of(context).primaryColor,
        onPressed: () {
          showSearch(context: context, delegate: JoinChat());
        },
        child: const Icon(Icons.add_comment_outlined),
      ),
      body: Consumer<ChatListProps>(
        builder: (context, data, child) {

          if(data.primaryLoading()) {
            return const Center(child: CircularProgressIndicator());
          }

          latestData = data;

          List<ChatTile> chats = [];
          for (var chatRoom in data.getRoomsResponse.sortBy(
              lastActivity: true,
              favoritesToTop: Provider.of<SettingsProvider>(context).val().talkSettings.sortFavoritesToTop)
          ) {
            chats.add(ChatTile(data: chatRoom, query: _query));
          }

          return RefreshIndicator(
            color: Theme.of(context).primaryColor,
            onRefresh: () {
              _query(renew: true);
              return Future.delayed(const Duration(seconds: 3));
            },
            child: ListView(children: chats),
          );
        },
      ),
    );
  }
}